/*
  IWT255.cpp - Tokyo Devices IWT255 Library 1.0.1
  http://tokyodevices.jp/
  Copyright (C) 2013-2014 I.W. Technology Firm, Inc.
*/

#include "Arduino.h"

#include "IWT255.h"
#include <Wire.h>

IWT255Impl::IWT255Impl()
{
}

void IWT255Impl::begin()
{
  Wire.begin();
}

void IWT255Impl::setDateTime(uint8_t data[], uint8_t length)
{
  // clock stop
  Wire.beginTransmission(RTC_SLAVE_ADDR);
  Wire.write(0x00);
  Wire.write(0x20);
  Wire.endTransmission();

  // Update registers
  Wire.beginTransmission(RTC_SLAVE_ADDR);
  Wire.write(RTC_REG_SEC_ADDR); // Alerm registers begins at 0x02
  Wire.write(data,length);
  Wire.endTransmission();

  // clock restart
  Wire.beginTransmission(RTC_SLAVE_ADDR);
  Wire.write(0x00);
  Wire.write(0x00);
  Wire.endTransmission();
}

uint8_t IWT255Impl::toBCD(uint8_t dec)
{
  return ( (dec/10) << 4 ) + (dec % 10);
}

void IWT255Impl::setDateTime(uint8_t year,
			     uint8_t month,
			     uint8_t day,
			     uint8_t wday,
			     uint8_t hour,
			     uint8_t min,
			     uint8_t sec)
{
  uint8_t data[7];
  data[0] = toBCD(sec);
  data[1] = toBCD(min);
  data[2] = toBCD(hour);
  data[3] = toBCD(day);
  data[4] = wday;
  data[5] = toBCD(month);
  data[6] = toBCD(year);
  setDateTime(data,7);
}

void IWT255Impl::startTimer()
{
  Wire.beginTransmission(RTC_SLAVE_ADDR);
  Wire.write(0x01); // Control_status_2 register
  Wire.write(0b00000010); // AF=0, AIE=1
  Wire.endTransmission();
}

void IWT255Impl::clearTimer()
{
  Wire.beginTransmission(RTC_SLAVE_ADDR);
  Wire.write(0x01); // Control_status_2 register
  Wire.write(0b00000000); // AF=0, AIE=0
  Wire.endTransmission();
}

void IWT255Impl::setHourTimer(bool b, uint8_t h)
{
  if( b == true ) {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x0A); // Alerm registers begins at 0x0A
    Wire.write(toBCD(h));       // AE_H Enabled
    Wire.endTransmission();
  } else {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x0A); // Alerm registers begins at 0x0A
    Wire.write(toBCD(h) | 0x80);  // AE_H Disabled
    Wire.endTransmission();
  }
}

void IWT255Impl::setMinuteTimer(bool b, uint8_t m)
{
  if( b == true ) {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x09); // Alerm registers begins at 0x09
    Wire.write(toBCD(m));        // AE_M Enabled
    Wire.endTransmission();
  } else {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x09); // Alerm registers begins at 0x09
    Wire.write(toBCD(m) | 0x80); // AE_M Disabled
    Wire.endTransmission();
  }
}

void IWT255Impl::setDayTimer(bool b, uint8_t d)
{
  if( b == true ) {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x0B); // Alerm registers begins at 0x0B
    Wire.write(toBCD(d));        // AE_D Enabled
    Wire.endTransmission();
  } else {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x0B); // Alerm registers begins at 0x0B
    Wire.write(toBCD(d) | 0x80); // AE_D Disabled
    Wire.endTransmission();
  }
}

void IWT255Impl::setWeekdayTimer(bool b, uint8_t w)
{
  if( b == true ) {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x0C); // Alerm registers begins at 0x0C
    Wire.write(toBCD(w));        // AE_W Enabled
    Wire.endTransmission();
  } else {
    Wire.beginTransmission(RTC_SLAVE_ADDR);
    Wire.write(0x0C); // Alerm registers begins at 0x0C
    Wire.write(toBCD(w) | 0x80); // AE_W Disabled
    Wire.endTransmission();
  }
}

void IWT255Impl::readDateTime()
{
  Wire.beginTransmission(RTC_SLAVE_ADDR);
  Wire.write(RTC_REG_SEC_ADDR);
  Wire.endTransmission();

  int rep = Wire.requestFrom(RTC_SLAVE_ADDR,7,true);

  if( rep == 7 ) {
    second = Wire.read() & 0x7F;
    minute = Wire.read() & 0x7F;
    hour = Wire.read() & 0x3F;
    day = Wire.read() & 0x3F;
    weekday = Wire.read() & 0x07;
    month = Wire.read() & 0x1F;
    year = Wire.read();
  }
}

uint8_t IWT255Impl::toDEC(uint8_t bcd)
{
  return (bcd >> 4) * 10 + (bcd & 0x0F);
}

uint8_t IWT255Impl::getYear()
{
  return toDEC(year);
}

uint8_t IWT255Impl::getMonth()
{
  return toDEC(month);
}

uint8_t IWT255Impl::getDay()
{
  return toDEC(day);
}

uint8_t IWT255Impl::getHour()
{
  return toDEC(hour);
}

uint8_t IWT255Impl::getMinute()
{
  return toDEC(minute);
}

uint8_t IWT255Impl::getSecond()
{
  return toDEC(second);
}

uint8_t IWT255Impl::getWeekday()
{
  return weekday;
}

void IWT255Impl::powerOff()
{
  Wire.beginTransmission(IWT255_SLAVE_ADDR);
  Wire.write(0x4c);
  Wire.endTransmission();
}

IWT255Impl IWT255 = IWT255Impl();
